// DataCollectionSink.cpp: implementation of the CDataCollectionSink class.
//
//////////////////////////////////////////////////////////////////////

#include "DataCollectionSink.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDataCollectionSink::CDataCollectionSink(string sLabel, int nTime):CPacketSink(sLabel,nTime)
{
	
}

CDataCollectionSink::~CDataCollectionSink()
{

}

bool CDataCollectionSink::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	if ((pIncommingPacket->Destination() != m_nParentNetAddress) && (m_nParentNetAddress != -1))
	{
		//LOG("Bad Routing at:" + m_sParentLabel + " - " + m_sLabel);
	}
	if (nConnectionLabel == 0)
	{
		m_nPacketCount++;
		pIncommingPacket->SetTimeFinishProcessing(m_nLocalTime);
		if (pIncommingPacket->PriorityClass() == 0)
		{
			m_vProcessedClassZero.push_back(pIncommingPacket);
		}
		if (pIncommingPacket->PriorityClass() == 1)
		{
			m_vProcessedClassOne.push_back(pIncommingPacket);
		}
		if (pIncommingPacket->PriorityClass() == 2)
		{
			m_vProcessedClassTwo.push_back(pIncommingPacket);
		}
		if (pIncommingPacket->PriorityClass() == 3)
		{
			m_vProcessedClassThree.push_back(pIncommingPacket);
		}
		if (pIncommingPacket->PriorityClass() == 4)
		{
			m_vProcessedClassFour.push_back(pIncommingPacket);
		}
		return true;
	}
	else
	{
		LOG("CPacketSink::AcceptPacket");
		return false;
	}
}

///////////////////////////////////////////////////////////////////////
// Data Collection
///////////////////////////////////////////////////////////////////////

/* Returns average time in system for the input vector */
double CDataCollectionSink::averageVectorTimeInSystem(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalTimeInSystem = 0;
	double nAverageTime = 0.0;
	for (int i = 0; i < vRequests.size(); i++)
	{
		pRequest = vRequests[i];
		nTotalTimeInSystem += pRequest->TotalTimeInSystem();
	}
	if (vRequests.size() > 0)
	{
		nAverageTime = (nTotalTimeInSystem / vRequests.size());
	}
	else
	{
		nAverageTime = 0.0;
	}
	return nAverageTime;
}

/* Returns average time in queue for the input vector */
double CDataCollectionSink::averageVectorTimeInQueue(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalTimeInQueue = 0.0;
	double nAverageTime = 0.0;

	for (int i = 0; i < vRequests.size(); i++)
	{
		pRequest = vRequests[i];
		nTotalTimeInQueue += pRequest->TotalTimeInQueue();
	}
	if (vRequests.size() > 0)
	{
		nAverageTime = (nTotalTimeInQueue / vRequests.size());
	}
	else
	{
		nAverageTime = 0.0;
	}
	return nAverageTime;
}

/* Returns average processing time for the input vector */
double CDataCollectionSink::averageVectorProcessingTime(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalProcessingTime = 0.0;
	double nAverageTime = 0.0;
	
	for (int i = 0; i < vRequests.size(); i++)
	{
		pRequest = vRequests[i];
		nTotalProcessingTime += pRequest->TotalProcessingTime();
	}
	if (vRequests.size() > 0)
	{
		nAverageTime = (nTotalProcessingTime / vRequests.size());
	}
	else
	{
		nAverageTime = 0.0;
	}
	return nAverageTime;
}

/* Returns average time in system for the input priority */
double CDataCollectionSink::AverageTimeInSystem(int nClass)
{
	switch (nClass)
	{
	case 0:
		return averageVectorTimeInSystem(m_vProcessedClassZero);
	case 1:
		return averageVectorTimeInSystem(m_vProcessedClassOne);
	case 2:
		return averageVectorTimeInSystem(m_vProcessedClassTwo);
	case 3:
		return averageVectorTimeInSystem(m_vProcessedClassThree);
	case 4:
		return averageVectorTimeInSystem(m_vProcessedClassFour);
	default:
		return 0.0;
	}
}

/* Returns average time in queue for the input priority */
double CDataCollectionSink::AverageTimeInQueue(int nClass)
{
		switch (nClass)
	{
	case 0:
		return averageVectorTimeInQueue(m_vProcessedClassZero);
	case 1:
		return averageVectorTimeInQueue(m_vProcessedClassOne);
	case 2:
		return averageVectorTimeInQueue(m_vProcessedClassTwo);
	case 3:
		return averageVectorTimeInQueue(m_vProcessedClassThree);
	case 4:
		return averageVectorTimeInQueue(m_vProcessedClassFour);
	default:
		return 0.0;
	}
}

/* Returns average processing time for the input priority */
double CDataCollectionSink::AverageProcessingTime(int nClass)
{
	switch (nClass)
	{
	case 0:
		return averageVectorProcessingTime(m_vProcessedClassZero);
	case 1:
		return averageVectorProcessingTime(m_vProcessedClassOne);
	case 2:
		return averageVectorProcessingTime(m_vProcessedClassTwo);
	case 3:
		return averageVectorProcessingTime(m_vProcessedClassThree);
	case 4:
		return averageVectorProcessingTime(m_vProcessedClassFour);
	default:
		return 0.0;
	}

}

/* Returns 95 Percentile processing time for the input priority */
double CDataCollectionSink::PercentileProcessingTime(int nClass)
{
	SortByProcessingTime(nClass);

	switch (nClass)
	{
	case 0:
		return percentileVectorProcessingTime(m_vProcessedClassZero);
	case 1:
		return percentileVectorProcessingTime(m_vProcessedClassOne);
	case 2:
		return percentileVectorProcessingTime(m_vProcessedClassTwo);
	case 3:
		return percentileVectorProcessingTime(m_vProcessedClassThree);
	case 4:
		return percentileVectorProcessingTime(m_vProcessedClassFour);
	default:
		return 0.0;
	}
}

/* Returns 95 Percentile total time in system for the input priority*/
double CDataCollectionSink::PercentileTotalTime(int nClass)
{
	SortByTotalTime(nClass);
	switch (nClass)
	{
	case 0:
		return percentileVectorTotalTime(m_vProcessedClassZero);
	case 1:
		return percentileVectorTotalTime(m_vProcessedClassOne);
	case 2:
		return percentileVectorTotalTime(m_vProcessedClassTwo);
	case 3:
		return percentileVectorTotalTime(m_vProcessedClassThree);
	case 4:
		return percentileVectorTotalTime(m_vProcessedClassFour);
	default:
		return 0.0;
	}
}

/* Returns 95 Percentile total time in system for the input priority*/
double CDataCollectionSink::PercentileQueueTime(int nClass)
{
	SortByQueueTime(nClass);
	switch (nClass)
	{
	case 0:
		return percentileVectorQueueTime(m_vProcessedClassZero);
	case 1:
		return percentileVectorQueueTime(m_vProcessedClassOne);
	case 2:
		return percentileVectorQueueTime(m_vProcessedClassTwo);
	case 3:
		return percentileVectorQueueTime(m_vProcessedClassThree);
	case 4:
		return percentileVectorQueueTime(m_vProcessedClassFour);
	default:
		return 0.0;
	}
}

/* Returns 95 Percentile total processing time for the input vector*/
double CDataCollectionSink::percentileVectorProcessingTime(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalProcessingTime = 0.0;
	double nTime = 0.0;

	if (vRequests.size() > 0)
	{
		int nIndex = ((vRequests.size()) / 100) * 95;
		pRequest = vRequests[nIndex];
		nTime = pRequest->TotalProcessingTime();
	}
	else
	{
		nTime = 0.0;
	}
	return nTime;
}

/* Returns 95 Percentile total time in system for the input vector*/
double CDataCollectionSink::percentileVectorTotalTime(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalProcessingTime = 0.0;
	double nTime = 0.0;

	if (vRequests.size() > 0)
	{
		int nIndex = ((vRequests.size()) / 100) * 95;
		pRequest = vRequests[nIndex];
		nTime = pRequest->TotalTimeInSystem();
	}
	else
	{
		nTime = 0.0;
	}
		return nTime;
}

/* Returns 95 Percentile total time in queue for the input vector*/
double CDataCollectionSink::percentileVectorQueueTime(vector<CPacket*> &vRequests)
{
	CPacket* pRequest = NULL;
	double nTotalProcessingTime = 0.0;
	double nTime = 0.0;

	if (vRequests.size() > 0)
	{
		int nIndex = ((vRequests.size()) / 100) * 95;
		pRequest = vRequests[nIndex];
		nTime = pRequest->TotalTimeInQueue();
	}
	else
	{
		nTime = 0.0;
	}
	return nTime;
}

/* Returns the total size of all processed requests */
int CDataCollectionSink::TotalSizeProcesed(int nClass)
{
	CPacket* pRequest = NULL;
	int nTotalSizeProcessed = 0;
	
	for (int i = 0; i < m_vProcessedClassZero.size(); i++)
	{
		pRequest = m_vProcessedClassZero[i];
		if ((nClass == 0) || (pRequest->PriorityClass() == nClass))
		{
			nTotalSizeProcessed += pRequest->PacketSize();
		}
	}
	return nTotalSizeProcessed;
}

/* returns count of all requests processed */
int CDataCollectionSink::TotalRequestsProcessed(int nClass)
{
	switch (nClass)
	{
	case 0:
		return m_vProcessedClassZero.size();
	case 1:
		return m_vProcessedClassOne.size();
	case 2:
		return m_vProcessedClassTwo.size();
	case 3:
		return m_vProcessedClassThree.size();
	case 4:
		return m_vProcessedClassFour.size();
	default:
		return 0;
	}
}

void CDataCollectionSink::CalculateTotals()
{
	m_nTotalProcessed += m_vProcessedClassZero.size();
}

int CDataCollectionSink::TotalProcessed()
{
	return m_nTotalProcessed;
}

////////////////////////////////////////////////////////////////////////
// Sort processed vectors
////////////////////////////////////////////////////////////////////////

/* Sort processed vector by total time */
void CDataCollectionSink::sortVectorByTotalTime(vector<CPacket*> &vRequests)
{
	int index;
    int gap;
    CPacket* pRequest;
	double nMagicNumber = 1.3;
    bool exchanged;
	int size = vRequests.size() - 1;
	gap = size;
	do 
	{
    	exchanged = false;
		gap = (int) (gap / nMagicNumber);
		if (gap == 0) gap = 1;
       	for (index = 0; index <= size - gap; index++) 
		{
        	if (vRequests[index]->TotalTimeInSystem() > vRequests[index + gap]->TotalTimeInSystem()) 
			{
				pRequest = vRequests[index];
				vRequests[index] = vRequests[index + gap];
				vRequests[index + gap] = pRequest;
            	exchanged = true;
           	}
        }
	} 
	while (exchanged || gap > 1);
}

/* Sort processed vector by queue time */
void CDataCollectionSink::sortVectorByQueueTime(vector<CPacket*> &vRequests)
{
	int index;
    int gap;
    CPacket* pRequest;
	double nMagicNumber = 1.3;
    bool exchanged;
	int size = vRequests.size() - 1;
	gap = size;
	do 
	{
    	exchanged = false;
		gap = (int) (gap / nMagicNumber);
		if (gap == 0) gap = 1;
       	for (index = 0; index <= size - gap; index++) 
		{
        	if (vRequests[index]->TotalTimeInQueue() > vRequests[index + gap]->TotalTimeInQueue()) 
			{
				pRequest = vRequests[index];
				vRequests[index] = vRequests[index + gap];
				vRequests[index + gap] = pRequest;
            	exchanged = true;
           	}
        }
	} 
	while (exchanged || gap > 1);
}

/* Sort processed vector by processing time */
void CDataCollectionSink::sortVectorByProcessingTime(vector<CPacket*> &vRequests)
{
	int index;
    int gap;
    CPacket* pRequest;
	double nMagicNumber = 1.3;
    bool exchanged;
	int size = vRequests.size() - 1;
	gap = size;
	do 
	{
    	exchanged = false;
		gap = (int) (gap / nMagicNumber);
		if (gap == 0) gap = 1;
       	for (index = 0; index <= size - gap; index++) 
		{
        	if (vRequests[index]->TotalProcessingTime() > vRequests[index + gap]->TotalProcessingTime()) 
			{
				pRequest = vRequests[index];
				vRequests[index] = vRequests[index + gap];
				vRequests[index + gap] = pRequest;
            	exchanged = true;
           	}
        }
	} 
	while (exchanged || gap > 1);
}

/* sort appropriate vector based on priority by processing time */
void CDataCollectionSink::SortByProcessingTime(int nClass)
{
	switch (nClass)
	{
	case 0:
		sortVectorByProcessingTime(m_vProcessedClassZero);
		break;
	case 1:
		sortVectorByProcessingTime(m_vProcessedClassOne);
		break;
	case 2:
		sortVectorByProcessingTime(m_vProcessedClassTwo);
		break;
	case 3:
		sortVectorByProcessingTime(m_vProcessedClassThree);
		break;
	case 4:
		sortVectorByProcessingTime(m_vProcessedClassFour);
		break;
	}
}


/* sort appropriate vector based on priority by queue time */
void CDataCollectionSink::SortByQueueTime(int nClass)
{
	switch (nClass)
	{
	case 0:
		sortVectorByQueueTime(m_vProcessedClassZero);
		break;
	case 1:
		sortVectorByQueueTime(m_vProcessedClassOne);
		break;
	case 2:
		sortVectorByQueueTime(m_vProcessedClassTwo);
		break;
	case 3:
		sortVectorByQueueTime(m_vProcessedClassThree);
		break;
	case 4:
		sortVectorByQueueTime(m_vProcessedClassFour);
		break;
	}
}

/* sort appropriate vector based on priority by total time */
void CDataCollectionSink::SortByTotalTime(int nClass)
{
	switch (nClass)
	{
	case 0:
		sortVectorByTotalTime(m_vProcessedClassZero);
		break;
	case 1:
		sortVectorByTotalTime(m_vProcessedClassOne);
		break;
	case 2:
		sortVectorByTotalTime(m_vProcessedClassTwo);
		break;
	case 3:
		sortVectorByTotalTime(m_vProcessedClassThree);
		break;
	case 4:
		sortVectorByTotalTime(m_vProcessedClassFour);
		break;
	}
}

void CDataCollectionSink::Report()
{
	cout<<"Total for Class 0:" + str(AverageTimeInSystem(0)) + "\n";
	cout<<"Total for Class 1:" + str(AverageTimeInSystem(1)) + "\n";
	cout<<"Total for Class 2:" + str(AverageTimeInSystem(2)) + "\n";
	cout<<"Total for Class 3:" + str(AverageTimeInSystem(3)) + "\n";
}

void CDataCollectionSink::Go()
{
	m_nLocalTime++;
}



