// DropTailQueue.h: interface for the CDropTailQueue class.
//
//
//
//
//                        _________________                            
//     FROM     Push --->|0               1|---> Pull     TO          
//    DEVICE             | DropTailQueue   |            DEVICE        
//                       |                 |                         
//                       |________2________|                          
//                                |
//                                |
//                                V 
//                               Push
//
//                               DROP
//                               TAIL
//
//////////////////////////////////////////////////////////////////////

#if !defined(DROPTAILQUEUE_H)
#define DROPTAILQUEUE_H

#include "Element.h"
#include "PushConnection.h"
#include "List.h"

class CDropTailQueue : public CElement  
{
public:
	CDropTailQueue(int nCapacity = 10000);
	virtual ~CDropTailQueue();

	//Virtual Finction Implementations
	bool AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel);
	CPacket* GetPacket(int nConnectionLabel);

	//Connections
	bool ConnectToSink(CElement* pElement, int nLabel);

	bool QueueFull();
	int BackPressure();
	void SetCapacity(int nCapacity = 10000);

	virtual bool Done();



protected:

	CPushConnection m_oPacketSink;
	CList<CPacket*> m_PacketQueue;
	double m_nQueueCapacity;
	double m_nCurrentUtilization;

		
};

#endif 
