// DynamicPriorityScheduler.cpp: implementation of the CDynamicPriorityScheduler class.
//
//////////////////////////////////////////////////////////////////////

#include "DynamicPriorityScheduler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDynamicPriorityScheduler::CDynamicPriorityScheduler(int nNumOfQueues)
{
	m_vClassifiedConnections.resize(nNumOfQueues);
	for (int i = 0; i < nNumOfQueues; i++)
	{
		CPullConnection* nNewConnection = new CPullConnection;
		m_vClassifiedConnections[i] = nNewConnection;
	}
	m_nNumberOfQueues = nNumOfQueues;
}

CDynamicPriorityScheduler::~CDynamicPriorityScheduler()
{

}

CPacket* CDynamicPriorityScheduler::GetPacket(int nConnectionLabel)
{
	int nQueue = 0;
	int nBestPriority = -1;
	int nNextPriority = -1;
	for (int i = 0; i < m_nNumberOfQueues; i++)
	{
		//Cast the connected element as a Weighted PriorityQueue
		CWeightedDropTailQueue* pQueue = (CWeightedDropTailQueue*)m_vClassifiedConnections[i]->Element();
		//Get the priority
		nNextPriority = pQueue->QueuePriority();

		if (nBestPriority < nNextPriority)
		{
			nBestPriority = nNextPriority;
			nQueue = i;
		}
	}

	CPacket* pReturnPacket = NULL; 
	pReturnPacket = m_vClassifiedConnections[nQueue]->Pull();
	return pReturnPacket;
}

bool CDynamicPriorityScheduler::ConnectToQueue(CElement* pElement, int nLabel, int nClassNum)
{
	if ((nClassNum >= 0) && (nClassNum <= m_nNumberOfQueues))
	{
		return m_vClassifiedConnections[nClassNum]->SetConnection(pElement, nLabel);
	}
	else
	{
		LOG("CDynamicPriorityScheduler::ConnectToQueue");
		return false;
	}
}

void CDynamicPriorityScheduler::SetNumberOfQueues(int nNumberOfQueues)
{
	
/*	for (int i = 0; i < m_vClassifiedConnections.size(); i++)
	{
		delete m_vClassifiedConnections[i];
	}*/

	m_vClassifiedConnections.resize(nNumberOfQueues);
	for (int i = 0; i < nNumberOfQueues; i++)
	{
		CPullConnection* nNewConnection = new CPullConnection;
		m_vClassifiedConnections[i] = nNewConnection;
	}
	m_nNumberOfQueues = nNumberOfQueues;
}
