// DynamicPriorityScheduler.h: interface for the CDynamicPriorityScheduler class.
//
//                         |    |    |    |    |
//                         |    |    |    |    |
//                         V    V    V    V    V
//                        Pull Pull Pull Pull Pull
//                         C1   C2   C3   C4   Cn
//                        _______________________         
//                       | 1     2    3    4    n|
//                       |   PacketClassifier    |
//                       |                       |
//                       |_______________________| 
//                                  |
//                                  | 
//                                  V                            
//                                 Pull
//////////////////////////////////////////////////////////////////////

#if !defined(DYNAMICPRIORITYSCHEDULER_H)
#define DYNAMICPRIORITYSCHEDULER_H


#include "Element.h"
#include "PullConnection.h"
#include "Packet.h"
#include <vector>
#include "WeightedDropTailQueue.h"

class CDynamicPriorityScheduler : public CElement  
{
public:
	CDynamicPriorityScheduler(int nNumOfQueues = 3);
	virtual ~CDynamicPriorityScheduler();

	CPacket* GetPacket(int nConnectionLabel);
	bool ConnectToQueue(CElement* pElement, int nLabel, int nClassNum);
	void SetNumberOfQueues(int nNumberOfQueues);

protected:
	vector<CPullConnection*> m_vClassifiedConnections;
	int m_nNumberOfQueues;

};

#endif // !defined(AFX_DYNAMICPRIORITYSCHEDULER_H__24AB73BC_A552_4AF2_AFB6_82EFD255EDF8__INCLUDED_)
