// EthernetDevice.cpp: implementation of the CEthernetDevice class.
//
//////////////////////////////////////////////////////////////////////

#include "EthernetDevice.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEthernetDevice::CEthernetDevice()
{
	m_pPacketProcessing = NULL;
	m_nPacketsToNetwork = 0;
	m_nPacketsFromNetwork = 0;
}

CEthernetDevice::~CEthernetDevice()
{

}


/////////////////////////////////////////////////////////////////////
// CPacket processing
/////////////////////////////////////////////////////////////////////

bool CEthernetDevice::ProcessBytes(int nBytes)
{

	int nBytesRemaining = nBytes;
	

	while (nBytesRemaining > 0)
	{
		//If there is no packet on the device, try to get one
		if (m_pPacketProcessing == NULL)
		{
			m_pPacketProcessing = m_oQueue3.Pull();
		}
		
		//If there is a packet to process
		if (m_pPacketProcessing != NULL)
		{
			//nBytesRemaining is decremented by the size of the packet
			nBytesRemaining -= m_pPacketProcessing->ProcessingRequirement();

			//The processing requirement of the packet is decremented 
			//by the speed of the link
			m_pPacketProcessing->DecrementProcessingRequirement(nBytes);

			//If the packet has been completely processed, send it on its way
			if (m_pPacketProcessing->ProcessingRequirement() <= 0)
			{
				m_pPacketProcessing->AppendHistory("TRANS:" + m_sParentLabel + "-" + m_sLabel);
				if (m_oNetwork2.Push(m_pPacketProcessing))
				{
					m_nPacketsToNetwork++;
					m_pPacketProcessing = NULL;
				}
				else
				{
					LOG("CEthernetDevice::ProcessBytes");
					LOG("Error pushing packet on to network");
					return false;
				}
			}
		}
		else
		{
			nBytesRemaining = 0;
		}
	}
	return true;
}

	
/////////////////////////////////////////////////////////////////////
// Virtual Function implementation 
/////////////////////////////////////////////////////////////////////

bool CEthernetDevice::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	if (nConnectionLabel == 0) //Packet pushed on to device from network
	{
		pIncommingPacket->AppendHistory("RCVD:" + m_sParentLabel + "-" + m_sLabel);
		m_nPacketsFromNetwork++;
		return m_oSystem1.Push(pIncommingPacket);
	}
	else
	{
		LOG("CEthernetDevice::AcceptPacket");
		return false; //This Element has no other push connections
	}
}

//////////////////////////////////////////////////////////////////////
// Setup Connections
//////////////////////////////////////////////////////////////////////

bool CEthernetDevice::ConnectToSystem(CElement* pElement, int nLabel)
{
	return m_oSystem1.SetConnection(pElement, nLabel);
}

bool CEthernetDevice::ConnectToNetwork(CElement* pElement, int nLabel)
{
	return m_oNetwork2.SetConnection(pElement, nLabel);
}

bool CEthernetDevice::ConnectToQueue(CElement* pElement, int nLabel)
{
	return m_oQueue3.SetConnection(pElement, nLabel);
}
void CEthernetDevice::Report()
{
	cout<<m_sParentLabel + " - " + m_sLabel + " Transmit: " + str(m_nPacketsToNetwork) + " \n";;
	cout<<m_sParentLabel + " - " + m_sLabel + " Receiv: " + str(m_nPacketsFromNetwork) + " \n";;
}

