//	EthernetDevice.h: interface for the CEthernetDevice class.
//	
//
//	An EthernetDevice is an element that has 4 connections.
//	Two outgoing push connections.  One incomming push connection, 
//	and one incomming pull connection.  It acts as an interface 
//	between the NetworkDevice and the Network.  
//                                                                           
//	                                                                         
//  N              _________________              S            
//  E    Push --->|0               1|---> Push    Y            
//  T             | EthernetDevice  |             S            
//  W    Push <---|2               3|<--- Pull    T            
//  O             |_________________|             E            
//  R                                             M            
//  K                                                          
//                                                             
//                                                             
//////////////////////////////////////////////////////////////////////

#if !defined(ETHERNETDEVICE_H)
#define ETHERNETDEVICE_H

#include "Element.h"
#include "DPIUtilities.h"
#include "PushConnection.h"
#include "PullConnection.h"

class CEthernetDevice : public CElement  
{
public:
	CEthernetDevice();
	virtual ~CEthernetDevice();

	//CPacket processing
	bool ProcessBytes(int nBytes); // Process nBytes of the current CPacket

	//Virtual Function implementation 
	bool AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel);
	
	//Setup Connections
	bool ConnectToSystem(CElement* pElement, int nLabel);
	bool ConnectToNetwork(CElement* pElement, int nLabel);
	bool ConnectToQueue(CElement* pElement, int nLabel);
	void Report();
	

protected:

	//Connections
	CPushConnection m_oSystem1;  //Connection 1
	CPushConnection m_oNetwork2; //Connection 2
	CPullConnection m_oQueue3;   //Connection 3

	int m_nPacketsToNetwork;
	int m_nPacketsFromNetwork;

	int m_nMAXConnectionSpeed; //The maximum speed of this device

	//CPacket processing
	CPacket* m_pPacketProcessing;  //Current CPacket being processed
};

#endif 
