//  List.h        Dennis Ippoliti        10/15/02
//
//  A templated List class.  Implements a linked list with 
//  specific behavior that I feel comfortable programming with
//
//  Behavior:  The interface includes methods to add and remove items at the
//  head and tail of the list.  It also includes methods to traverse the list
//  you can move to the head, tail, next, prev, and first of the list.
//
//  Movehead will move the current and position pointers to the first item
//  in the list.  MoveFirst will move the current pointer to the head
//  of the list, but will move the position pointer to before the head
//  of list.  Doing this enables me to call the Movefirst method and then 
//  traverse the entire list using only Getnext or MoveNext.  Using GetNext
//  after Movefirst returns the first item in the list.  Using Getnext
//  after using Movehead returns the second item in the list. 
//
//  The interface also provides methods to return the size of the list,
//  get and set the current position of the list, determine End of List (EOl)
//  and Beginning of List (BOL)
/////////////////////////////////////////////////////////////////////////////

#ifndef _CLIST
#define _CLIST

//NULL is a constant defined in various include files
//I define it here incase I do not use those files
#ifndef NULL
#ifdef  __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

//////////////////////////////////////
//
// CNode.  
// Internal Class for storing each item
//
//////////////////////////////////////
template <class Type>
class CNode                         
{                                   
  public:
    Type item;
    CNode * next;
	CNode * prev;
};

///////////////////////////////////////////////////////////////////////////////
//
// CList  
// This class is like an SUV.  It does everything, but is not fast and sleek.
// It is big and bulky.  I have found it very useful, but ineffecient.  One
// of these days, if I ever find the time, I will tweek it to be better code
//
///////////////////////////////////////////////////////////////////////////////
template <class Type>
class CList
{
  public:
	//Construction / Destruction
    CList();                                            // constructor
    CList(const CList<Type> & q);                       // copy constructor
    ~CList();                                           // destructor
    

	//Overloaded / Custom Operators
		CList<Type> & operator=(const CList<Type> & pList); // override assignment operator

	//Add items to list
	void AddHead(const Type & item);                    // adds item to head of Clist
    void AddTail(const Type & item);                    // adds item to tail of CList
	void InsertAfter(const Type & item);				// insterts item after current
	void InsertBefore(const Type & item);				// insterts item before current
	
	//Remove items from the list
	void RemoveHead();                                  // removes item at head of CList
    void RemoveTail();                                  // removes item at tail of CList
    void RemoveAll();									// removes all items from the list
	void RemoveCurrent();								// removes current item from list
	void RemoveNext();									// removes next item in list
	void RemovePrev();									// removes prev item in list
														
	/*Sequential Searches.  
	  List is not the best structure
	  if you need to consistantly
	  add, sort and search*/
	bool FindFirst(const Type & item);
	bool FindFirst(const int & nIntValue);
	bool FindNext(const Type & item);
	bool FindPrev(const Type & item);

	//Sorting using QuickSort 
	void SortAscending();
	void SortDescending();
	void SortReferencesAscending();
	void SortReferencesDescending();

	//Traverse the list
	void MoveFirst();                                   // Sets current position head of list (ready to read inorder)
    void MoveHead();                                    // Sets current position head of list
    void MoveTail();                                    // Sets current position to tail of list
    void MoveNext();                                    // Sets current position to next item in list
    void MovePrev();                                    // Sets current position to previous item in list

	//Access items in the list
	Type & GetHead();     	                            // returns item at head of CList
    Type & GetTail() const;	                            // returns item at tail of CList
    Type & GetCurrent();                                // returns current item of CList
    Type & GetNext();	                                // returns next item of CList
	Type & GetPrev();	                                // returns prev item CList
	Type & PeekNext();	                                // returns next item but does not advance counter
	Type & PeekPrev();	                                // returns prev item but does not advance counter

	//Get and set list status
	bool IsEmpty() const;                               // returns if CList is empty
    int  GetListSize() const;                            // returns value of nListSize
	int  GetPosition() const;							// returns integer value representing current position in the list
	void SetPosition(int nNewPosition);					// sets position based on integer value
	bool EOL();											// Returns true if Current record is the last record
    bool BOL();											// Returns true if the current record is the first record
														// also sets current pointer to found record
  private:

	//Private member variables
	CNode<Type> * pHead;                                // pointer to first item in list
    CNode<Type> * pTail;		                        // pointer to last item in list
    CNode<Type> * pCurrent;                             // pointer to current item in list
    CNode<Type> * CopyList(const CNode<Type> * ptr);    // Copies the list returns pointer to head of new list
	int nListSize;							            // number of items in CList
	int nPosition;							            // integer representation of position in the list

	//Private member functions
	void QuickSort(CNode<Type>* pLow, CNode<Type>*		//QuickSort.  Accomplishes Ascending and Decending Sort
		          pHigh, bool bIsAscending = true);
	void QuickSortReferences(CNode<Type>* pLow, CNode<Type>*		//QuickSort.  Accomplishes Ascending and Decending Sort
		          pHigh, bool bIsAscending = true);
	CNode<Type>* partition(CNode<Type>** pLow,			//Partition function for QuickSort
		                   CNode<Type>** pHigh);
	CNode<Type>* partitionDecending(CNode<Type>** pLow, //Partition function for QuickSort
		                          CNode<Type>** pHigh);
	CNode<Type>* partitionReferences(CNode<Type>** pLow,			//Partition function for QuickSort
		                   CNode<Type>** pHigh);
	CNode<Type>* partitionReferencesDecending(CNode<Type>** pLow, //Partition function for QuickSort
		                          CNode<Type>** pHigh);
};

////////////////////////////////////////////////////////////////////////////////////////



//////////////////////////////////////
//
// Construction / Destruction
//
//////////////////////////////////////
template <class Type>
CList<Type>::CList()
// Constructor: initializes CList to be empty (pHead = pTail = pCurrent = NULL, nPosition = 0)
{
    pHead = NULL;
    pTail = NULL;
    pCurrent = NULL;
    nListSize = 0;
	nPosition = 0;
}

template <class Type>
CList<Type>::CList(const CList<Type> & pList)
// Constructor: initializes CList with another CList
{
    nListSize = 0;
	pHead = CopyList(pList.pHead);
    pTail = pHead;
	while (pTail != NULL && pTail->next != NULL)
	{
        pTail = pTail->next;
	}
      pCurrent = pHead;
	  if (pHead == NULL)
	  {
		  nPosition = 0;
	  }
	  else
	  {
		  nPosition = 1;
	  }
}

template <class Type>
CList<Type>::~CList()
// Destructor: deallocates nodes in the linked list
{
    RemoveAll();
}

//////////////////////////////////////
//
// Overloaded / Custom Operators
//
//////////////////////////////////////
template <class Type>
CList<Type> & CList<Type>::operator=(const CList<Type> & pList)
// Assignment operator
{
    if (this != &pList)
    {
        RemoveAll();
        nListSize = 0;
		pTail = CopyList(pList.pHead);
		pHead = pTail;
		while (pHead != NULL && pHead->prev != NULL)
		{
			pHead = pHead->prev;
		}
    }
    return *this;
}

//////////////////////////////////////
//
// Add Items to the List
//
//////////////////////////////////////
template <class Type>
void CList<Type>::AddHead(const Type & newNode)
// Adds an item to the head of the list.  If list 
// is not empty, pCurrent and nPosition pointers continue
// to represent the item they did before the method call
{
    CNode<Type> * temp = new CNode<Type>;
    temp->item = newNode;
    temp->next = pHead;
	temp->prev = NULL;
	if (nListSize > 0)
	{
		pHead->prev = temp;
	}
	
    pHead = temp;
	if (pCurrent == NULL)
	{
		pCurrent = pHead;
		nPosition = 1;
	}
    
    if (pTail == NULL)
    {
        pTail = temp;
    }
	if (nPosition > 0)
	{
		nPosition++;
	}
	else
	{
		pCurrent = pHead;
	}
    nListSize++;
}

template <class Type>
void CList<Type>::AddTail(const Type & newNode)
// Adds an item to the tail of the list.  If list 
// is not empty, pCurrent and nPosition pointers continue
// to represent the item they did before the method call
{

	
	CNode<Type> * temp = new CNode<Type>;
    temp->item = newNode;
	temp->next = NULL;
	temp->prev = pTail;
	if (pTail == NULL)
	{
		pHead = temp;
		pCurrent = temp;
		pTail = temp;
		nPosition = 1;
	}
	else
	{
		pTail->next = temp;
		pTail = temp;    
		if (pCurrent == NULL)
		{
			pCurrent = pTail;
			nPosition = 1;
		}
	}
    nListSize++;
}

template <class Type>
void CList<Type>::InsertBefore(const Type & newNode)
// Adds an item before the current node.  If list 
// is not empty, pCurrent and nPosition pointers continue
// to represent the item they did before the method call
{
    if (pCurrent == pHead)
	{
		AddHead(newNode);
	}
	else if (pCurrent == NULL)
	{
		AddHead(newNode);
	}
	else
	{
		CNode<Type> * temp = new CNode<Type>;
		temp->item = newNode;
		temp->next = pCurrent;
		temp->prev = pCurrent->prev;
		pCurrent->prev->next = temp;
		pCurrent->prev = temp;
		
			
		if (nPosition > 0)
		{
			nPosition++;
		}
		if (pCurrent == pHead)
		{
			pHead = temp;
		}
		nListSize++;
	}
}

template <class Type>
void CList<Type>::InsertAfter(const Type & newNode)
// Inserts an item after the current item.  If list 
// is not empty, pCurrent and nPosition pointers continue
// to represent the item they did before the method call
{
    if (pCurrent == NULL)
	{
		AddHead(newNode);
	}
	else if (pCurrent->next == NULL)
	{
		AddTail(newNode);
	}
	else
	{
		CNode<Type> * temp = new CNode<Type>;
		temp->item = newNode;
		temp->next = pCurrent->next;
		temp->prev = pCurrent;
		pCurrent->next = temp;
	
		if (nPosition > 0)
		{
			nPosition++;
		}
		if (pCurrent == pTail)
		{
			pTail = temp;
		}
		nListSize++;
	}
}


//////////////////////////////////////
//
// Remove Items From the List
//
//////////////////////////////////////
template <class Type>
void CList<Type>::RemoveHead()
// Removes item from head of the list.  Adjusts the pCurrent,
// nPosition, nListSize indicators
{
    CNode<Type> * temp = pHead;
    pHead = pHead->next;
	if (pHead != NULL)
		pHead->prev = NULL;
    if (pCurrent == temp)
    {
       pCurrent = pHead;
    }

	delete temp;
	nPosition--;

	if (nPosition <= 0)
	{
		nPosition = 0;
	}
	if (pHead == NULL)
    {
        pTail = NULL;
		nPosition = 0;
    }

    nListSize--;

    if (nListSize < 0)
    {
		nListSize = 0;
    }
	if (pHead == NULL)
		nListSize = 0;
}

template <class Type>
void CList<Type>::RemoveTail()
// Removes item from head of the list.  Adjusts the pCurrent,
// nPosition, nListSize indicators
{
    CNode<Type> * temp = pTail;
	if (pTail)
	{
		pTail = pTail->prev;
		pTail->next = NULL;
	}
    
    if (pCurrent == temp)
    {
       pCurrent = pTail;
	   nPosition--;
    }

    delete temp;
    if (pTail == NULL)
    {
        pHead = NULL;
		nPosition = 0;
    }

    nListSize--;

    if (nListSize < 0)
    {
		nListSize = 0;
    }

	if (pHead == NULL)
		nListSize = 0;
}

template <class Type>
void CList<Type>::RemoveNext()
// Removes next item in list
{
    CNode<Type> * pTemp = pCurrent->next;
	if (pCurrent->next == pTail)
	{
		RemoveTail();
	}
	else if (pCurrent != pTail)
	{
		pCurrent->next = pCurrent->next->next;
		pCurrent->next->prev = pCurrent;
		delete pTemp;
		nListSize--;
	}
	if (pHead == NULL)
		nListSize = 0;
}

template <class Type>
void CList<Type>::RemovePrev()
// Removes prev item in list
{
    CNode<Type> * pTemp = pCurrent->prev;
	if (pCurrent->next == pHead)
	{
		RemoveHead();
	}
	else if (pCurrent != pHead)
	{
		pCurrent->prev = pCurrent->prev->prev;
		pCurrent->prev->next = pCurrent;
		delete pTemp;
		nListSize--;
	}
	if (pHead == NULL)
		nListSize = 0;
}

template <class Type>
void CList<Type>::RemoveCurrent()
// Removes current item from the list.  Adjusts the pCurrent,
// nPosition, nListSize indicators
{
    
	if (pCurrent == pHead)
	{
		RemoveHead();
	}
	else
	{
		MovePrev();
		RemoveNext();
	}
	if (pHead == NULL)
		nListSize = 0;
}

template <class Type>
void CList<Type>::RemoveAll()
// Removes all items from the list.
{
    while (!IsEmpty())
    {
        RemoveHead();
    }
	if (pHead == NULL)
		nListSize = 0;
}


//////////////////////////////////////
//
// Search the list.  
// These Searches are sequential.
//
//////////////////////////////////////

template <class Type>
bool CList<Type>::FindFirst(const Type & item)
//Finds the first item in the list that
//Matches the search criteria
{
    bool bIsFound = false;
	if (nListSize > 0)
	{
		MoveFirst();
		while (!EOL())
		{
			if (GetNext() == item)
			{
				bIsFound = true;
				break;
			}
		}
	}
	return bIsFound;
}

template <class Type>
bool CList<Type>::FindFirst(const int & nIntValue)
//Finds the first item in the list that
//Matches the search criteria
{
    bool bIsFound = false;
	if (nListSize > 0)
	{
		MoveFirst();
		while (!EOL())
		{
			if (GetNext() == nIntValue)
			{
				bIsFound = true;
				break;
			}
		}
	}
	return bIsFound;
}

template <class Type>
bool CList<Type>::FindNext(const Type & item)
//Finds the next item in the list
//That matches the search criteria
{
    bool bIsFound = false;
	if (nListSize > 0)
	{
		while (!EOL())
		{
			if (GetNext() == item)
			{
				bIsFound = true;
				break;
			}
		}
	}
	return bIsFound;
}

template <class Type>
bool CList<Type>::FindPrev(const Type & item)
//Finds the previous item in the list
//that matches the search criteria
{
    bool bIsFound = false;
	if (nListSize > 0)
	{
		while (!BOL())
		{
			if (GetPrev() == item)
			{
				bIsFound = true;
				break;
			}
		}
	}
	return bIsFound;
}

//////////////////////////////////////
//
// Sorting Functions
//
//////////////////////////////////////

template <class Type>
CNode<Type>* CList<Type>::partition(CNode<Type>** pLow, CNode<Type>** pHigh)
//Partitions the list.  Small nodes on the left, Large on the right
//
// Very Ugly Function.  To Do:  Clean this up!
{
	CNode<Type>* pPivot = *pLow;
	CNode<Type>* pBottom = *pLow;
	CNode<Type>* pTop = *pHigh;
	CNode<Type>* pCompare = NULL;
	CNode<Type>* pTemp = pBottom->next;
	CNode<Type>* pSaveBottom = pBottom->prev;
	CNode<Type>* pSaveTop = pTop->next;
	
	while ( (pCompare != *pHigh) && (pTemp != NULL))
	{
		pCompare = pTemp;
		pTemp = pTemp->next;
		if (pCompare != NULL)
		{
			if (pCompare->item <= pPivot->item)
			{
				if (pCompare != pBottom)
				{
					if (pCompare != pTop)
						pCompare->next->prev = pCompare->prev;
					else
						pTop = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->prev = pBottom->prev;
					pCompare->next = pBottom;
					pBottom->prev = pCompare;
					pBottom = pCompare;
					pPivot->next = pTemp;
				}
			}
			else
			{
				if (pCompare != pTop)
				{
					pCompare->next->prev = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->next = pTop->next;
					pCompare->prev = pTop;
					pTop->next = pCompare;
					pTop = pCompare;
					pPivot->next = pTemp;
				}
			}
		}
	}
	*pLow = pBottom;
	*pHigh = pTop;
	if (pSaveBottom != NULL)
		pSaveBottom->next = pBottom;
	else
		pHead = pBottom;
	if (pSaveTop != NULL)
		pSaveTop->prev = pTop;
	else
		pTail = pTop;

	return pPivot;
}

template <class Type>
CNode<Type>* CList<Type>::partitionDecending(CNode<Type>** pLow, CNode<Type>** pHigh)
//Partitions the list.  Large nodes on the left, Small on the right
//
// Very Ugly Function.  To Do:  Clean this up!
{
	CNode<Type>* pPivot = *pLow;
	CNode<Type>* pBottom = *pLow;
	CNode<Type>* pTop = *pHigh;
	CNode<Type>* pCompare = NULL;
	CNode<Type>* pTemp = pBottom->next;
	CNode<Type>* pSaveBottom = pBottom->prev;
	CNode<Type>* pSaveTop = pTop->next;
	
	while ( (pCompare != *pHigh) && (pTemp != NULL))
	{
		pCompare = pTemp;
		pTemp = pTemp->next;
		if (pCompare != NULL)
		{
			if (pCompare->item >= pPivot->item)
			{
				if (pCompare != pBottom)
				{
					if (pCompare != pTop)
						pCompare->next->prev = pCompare->prev;
					else
						pTop = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->prev = pBottom->prev;
					pCompare->next = pBottom;
					pBottom->prev = pCompare;
					pBottom = pCompare;
					pPivot->next = pTemp;
				}
			}
			else
			{
				if (pCompare != pTop)
				{
					pCompare->next->prev = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->next = pTop->next;
					pCompare->prev = pTop;
					pTop->next = pCompare;
					pTop = pCompare;
					pPivot->next = pTemp;
				}
			}
		}
	}
	*pLow = pBottom;
	*pHigh = pTop;
	if (pSaveBottom != NULL)
		pSaveBottom->next = pBottom;
	else
		pHead = pBottom;
	if (pSaveTop != NULL)
		pSaveTop->prev = pTop;
	else
		pTail = pTop;

	return pPivot;
}

template <class Type>
CNode<Type>* CList<Type>::partitionReferences(CNode<Type>** pLow, CNode<Type>** pHigh)
//Partitions the list.  Small nodes on the left, Large on the right
//
// Very Ugly Function.  To Do:  Clean this up!
{
	CNode<Type>* pPivot = *pLow;
	CNode<Type>* pBottom = *pLow;
	CNode<Type>* pTop = *pHigh;
	CNode<Type>* pCompare = NULL;
	CNode<Type>* pTemp = pBottom->next;
	CNode<Type>* pSaveBottom = pBottom->prev;
	CNode<Type>* pSaveTop = pTop->next;
	
	while ( (pCompare != *pHigh) && (pTemp != NULL))
	{
		pCompare = pTemp;
		pTemp = pTemp->next;
		if (pCompare != NULL)
		{
			if (*(pCompare->item) <= *(pPivot->item))
			{
				if (pCompare != pBottom)
				{
					if (pCompare != pTop)
						pCompare->next->prev = pCompare->prev;
					else
						pTop = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->prev = pBottom->prev;
					pCompare->next = pBottom;
					pBottom->prev = pCompare;
					pBottom = pCompare;
					pPivot->next = pTemp;
				}
			}
			else
			{
				if (pCompare != pTop)
				{
					pCompare->next->prev = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->next = pTop->next;
					pCompare->prev = pTop;
					pTop->next = pCompare;
					pTop = pCompare;
					pPivot->next = pTemp;
				}
			}
		}
	}
	*pLow = pBottom;
	*pHigh = pTop;
	if (pSaveBottom != NULL)
		pSaveBottom->next = pBottom;
	else
		pHead = pBottom;
	if (pSaveTop != NULL)
		pSaveTop->prev = pTop;
	else
		pTail = pTop;

	return pPivot;
}

template <class Type>
CNode<Type>* CList<Type>::partitionReferencesDecending(CNode<Type>** pLow, CNode<Type>** pHigh)
//Partitions the list.  Large nodes on the left, Small on the right
//
// Very Ugly Function.  To Do:  Clean this up!
{
	CNode<Type>* pPivot = *pLow;
	CNode<Type>* pBottom = *pLow;
	CNode<Type>* pTop = *pHigh;
	CNode<Type>* pCompare = NULL;
	CNode<Type>* pTemp = pBottom->next;
	CNode<Type>* pSaveBottom = pBottom->prev;
	CNode<Type>* pSaveTop = pTop->next;
	
	while ( (pCompare != *pHigh) && (pTemp != NULL))
	{
		pCompare = pTemp;
		pTemp = pTemp->next;
		if (pCompare != NULL)
		{
			if (*(pCompare->item) >= *(pPivot->item))
			{
				if (pCompare != pBottom)
				{
					if (pCompare != pTop)
						pCompare->next->prev = pCompare->prev;
					else
						pTop = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->prev = pBottom->prev;
					pCompare->next = pBottom;
					pBottom->prev = pCompare;
					pBottom = pCompare;
					pPivot->next = pTemp;
				}
			}
			else
			{
				if (pCompare != pTop)
				{
					pCompare->next->prev = pCompare->prev;
					pCompare->prev->next = pCompare->next;
					pCompare->next = pTop->next;
					pCompare->prev = pTop;
					pTop->next = pCompare;
					pTop = pCompare;
					pPivot->next = pTemp;
				}
			}
		}
	}
	*pLow = pBottom;
	*pHigh = pTop;
	if (pSaveBottom != NULL)
		pSaveBottom->next = pBottom;
	else
		pHead = pBottom;
	if (pSaveTop != NULL)
		pSaveTop->prev = pTop;
	else
		pTail = pTop;

	return pPivot;
}

template <class Type>
void CList<Type>::QuickSort(CNode<Type>* pLow, CNode<Type>* pHigh, bool bIsAscending)
//QuickSort algorithm.  Will sort Ascending or Decending based on the input variable
{
	CNode<Type>* pIndex = NULL;
	if ((nListSize>1) && (pLow != pHigh))
	{	
		if (bIsAscending)
			pIndex = partition(&pLow, &pHigh);
		else
			pIndex = partitionDecending(&pLow, &pHigh);

		if (pLow != pIndex)
		{
			QuickSort(pLow, pIndex->prev, bIsAscending);
		}
		if (pHigh != pIndex)
		{
			QuickSort(pIndex->next, pHigh, bIsAscending);
		}
		
	}
}

template <class Type>
void CList<Type>::QuickSortReferences(CNode<Type>* pLow, CNode<Type>* pHigh, bool bIsAscending)
//QuickSort algorithm.  Will sort Ascending or Decending based on the input variable
{
	CNode<Type>* pIndex = NULL;
	if ((nListSize>1) && (pLow != pHigh))
	{	
		if (bIsAscending)
			pIndex = partitionReferences(&pLow, &pHigh);
		else
			pIndex = partitionReferencesDecending(&pLow, &pHigh);

		if (pLow != pIndex)
		{
			QuickSortReferences(pLow, pIndex->prev, bIsAscending);
		}
		if (pHigh != pIndex)
		{
			QuickSortReferences(pIndex->next, pHigh, bIsAscending);
		}
		
	}
}

//Public Sorting Functions
template <class Type>
void CList<Type>::SortAscending()
{
	QuickSort(pHead, pTail);
}
template <class Type>
void CList<Type>::SortDescending()
{
	QuickSort(pHead, pTail, false);
}

template <class Type>
void CList<Type>::SortReferencesAscending()
{
	QuickSortReferences(pHead, pTail);
}
template <class Type>
void CList<Type>::SortReferencesDescending()
{
	QuickSortReferences(pHead, pTail, false);
}

//////////////////////////////////////
//
// Traverse The List
//
//////////////////////////////////////
template <class Type>
void CList<Type>::MoveFirst() 
// Assumes list is not empty
// Moves current indicator to first item in list
// Sets position to 0
{
    pCurrent = pHead;
	nPosition = 0;
}

template <class Type>
void CList<Type>::MoveHead() 
// Moves current indicator to first item in list
// Sets position to 1
{
    pCurrent = pHead;
	if (pHead == NULL)
	{
		nPosition = 0;
	}
	else
	{
		nPosition = 1;
	}
}

template <class Type>
void CList<Type>::MoveTail() 
// Moves current indicator to last item in list
// Sets position to 0 if list empty, otherwise to nListSize
{
    pCurrent = pTail;
	if (pTail == NULL)
	{
	  nPosition = 0;
	}
	else
	{
	  nPosition = nListSize;
	}
}

template <class Type>
void CList<Type>::MoveNext() 
// Assumes: List is not empty. List is not at EOL
// Advances position and current indicators
{
    if ((nPosition == 0)  && (nListSize > 0))
	{
		nPosition = 1;
	}
	else
	{
		pCurrent = pCurrent->next;
		nPosition++;
		if (pCurrent == NULL)
		{
			pCurrent = pTail;
			if (pTail == NULL)
			{
	 			nPosition = 0;
			}
			else
			{
 			nPosition = nListSize;
			}
		}
	}
}

template <class Type>
void CList<Type>::MovePrev() 
// Assumes: List is not empty. List is not at BOL
// Retards position and current indicators
{
    if (pCurrent == pHead)
	{
		nPosition = 0;
	}
	else
	{
		pCurrent = pCurrent->prev;
		nPosition--;
	}
}

//////////////////////////////////////
//
// Access Items in the List
//
//////////////////////////////////////
template <class Type>
Type & CList<Type>::GetHead()
// Assumes: List is not empty.
// Returns: Item at head of list
// If nPosition is 0 sets nPosition to 1 
// so the next Getnext or Movenext call 
// will return second item of list.
{
	if (nPosition == 0)
	{
		nPosition = 1;
	}
    return pHead->item;
}

template <class Type>
Type & CList<Type>::GetTail() const
// Assumes: List is not empty
// Returns: Last item of list
{
    return pTail->item;
}

template <class Type>
Type & CList<Type>::GetCurrent()
// Assumes: List is not empty.
// Returns: Current item in list
// If nPosition is 0 sets nPosition to 1 
// so the next Getnext or Movenext call 
// will return second item of list.
{
    if (nPosition == 0)
	{
		nPosition = 1;
	}
	return pCurrent->item;
}

template <class Type>
Type & CList<Type>::GetNext() 
// Assumes: List is not empty. List is not at EOL
// Returns: Next item in list
// Advances position and current indicators
// If position is 0, works with item 1
// If position is 1, works with item 2
{
    if ((nPosition == 0) && (pHead != NULL))
	{
		nPosition = 1;
		return pHead->item;
	}
	else
	{
		CNode<Type> * temp = pCurrent;
		pCurrent = pCurrent->next;
		nPosition++;
		if (pCurrent == NULL)
		{
			pCurrent = pTail;
			if (nPosition > nListSize)
			{
				nPosition = nListSize;
			}
		}
		return temp->next->item;
	}
}

template <class Type>
Type & CList<Type>::GetPrev() 
// Assumes: List is not empty. List is not at BOL
// Returns: Previous item in list
// Retards position and current indicators
{
    MovePrev();
	return pCurrent->item;
}

template <class Type>
Type & CList<Type>::PeekNext() 
// Assumes: List is not empty. List is not at EOL
// Returns: Next item in list
// Advances position and current indicators
// If position is 0, works with item 1
// If position is 1, works with item 2
{
    if ((nPosition == 0) && (pHead != NULL))
	{
		return pHead->item;
	}
	else
	{
		return pCurrent->next->item;
	}
}

template <class Type>
Type & CList<Type>::PeekPrev() 
// Assumes: List is not empty. List is not at EOL
// Returns: Next item in list
// Advances position and current indicators
// If position is 0, works with item 1
// If position is 1, works with item 2
{

	return pCurrent->prev->item;

}

//////////////////////////////////////
//
// Get / Set List Status
//
//////////////////////////////////////
template <class Type>
bool CList<Type>::IsEmpty() const
// Returns: true if empty CList, else false
{
    return (pHead == NULL);
}

template <class Type>
int CList<Type>::GetListSize() const
// Returns: number of items in the list
{
    return nListSize;
}

template <class Type>
int CList<Type>::GetPosition() const
// Returns: current position in list
{
    return nPosition;
}

template <class Type>
void CList<Type>::SetPosition(int iNewPosition)
// Sets position based on input.  If position
// is out of range, sents position to last item in list
{
    if (iNewPosition >= nListSize)
	{
		nPosition = nListSize;
		pCurrent = pTail;
	}
	else if (iNewPosition <= 0)
	{
		nPosition = 0;
		pCurrent = pHead;
	}
	else
	{
		pCurrent = pHead;
		for (int i = 1; i < iNewPosition; i++)
		{
			pCurrent = pCurrent->next;
		}
		nPosition = iNewPosition;
	}
}

template <class Type>
bool CList<Type>::EOL() 
//Returns true if pCurrent is the last item 
//in list and nPosition is not 0
//If pCurrent == pHead and nPosition == 0 then 
//List contains only one item and is at BOL
{
    if (nListSize < 1)
	{
		return true;
	}
	else if (nPosition == 0)
	{
		return false;
	}
	else
	{
		return (pCurrent == pTail);
	}
}

template <class Type>
bool CList<Type>::BOL() 
//Returns true if nPosition is 0
{
    return (nPosition == 0);
}

template <class Type>
CNode<Type> * CList<Type>::CopyList(const CNode<Type> * ptr)
// private member function to recursively copy a linked list
{
    if (ptr == NULL) 
	{
		return NULL;
	}
	else
	{
		CNode<Type> * pNewNode = new CNode<Type>;
		CNode<Type> * pTempNode;
		pNewNode->prev = NULL;
		pTempNode = pNewNode;
		nListSize++;
		pTempNode->item = ptr->item;
		while (ptr->next !=NULL)
		{
			pNewNode = new CNode<Type>;
			pNewNode->prev = pTempNode;
			pTempNode->next = pNewNode;
			pTempNode = pNewNode;
			nListSize++;
			ptr = ptr->next;
			pTempNode->item = ptr->item;
		}
		pTempNode->next = NULL;
		return pTempNode;
	}
}

#endif
 
