#include "Link.h"
#include "PacketGenerator.h"
#include "StoreAndForwardRepeater.h"
#include "DestinationComponent.h"
#include "TCPDestinationComponent.h"
#include "Router.h"
#include "TCPSource.h"
#include "TCPTrafficGenerator.h"

void ConnectDevices(CComponent& oSourceDevice, CComponent& oDestinationDevice, CLink* pOutLink, CLink* oInLink, int nSourceDevice, int nDestinationDevise);

int main ( void )
{
	const int NUM_OF_LINKS = 7;
	//1 Traffic Generators
	CTCPTrafficGenerator oTCPSource1("TCPSource1",0,1, TCP);

	//4 Routers
	CRouter oMultiPathRouter1("RRRouter1", 4, SELECTIVE_MULTI_PATH);
	CRouter oSinglePathRouter1("SingleRouter1", 2, SINGLE_PATH);
	CRouter oSinglePathRouter2("SingleRouter2", 2, SINGLE_PATH);
	CRouter oSinglePathRouter3("SingleRouter3", 2, SINGLE_PATH);

	oMultiPathRouter1.SetPressureLimit(1,100);
	oMultiPathRouter1.SetPressureLimit(2,100);
	oMultiPathRouter1.SetPressureLimit(3,100);

	
	//1 Sinks
	CTCPDestinationComponent oDestination1("TCPSink1", 3, 1);

	//5 OutBound Links
	CLink oOutLinks[NUM_OF_LINKS];

	//5 Inbound Links
	CLink oInLinks[NUM_OF_LINKS];

	
	for (int i = 0; i < NUM_OF_LINKS; i++)
	{
		oOutLinks[i].SetConnectionSpeed(5000);
		oInLinks[i].SetConnectionSpeed(5000);
	}

	oOutLinks[0].SetConnectionSpeed(20000);
	oInLinks[0].SetConnectionSpeed(20000);

	/*oOutLinks[2].SetConnectionSpeed(1000);
	oInLinks[2].SetConnectionSpeed(1000);

	oOutLinks[5].SetConnectionSpeed(1000);
	oInLinks[5].SetConnectionSpeed(1000);

	oOutLinks[3].SetConnectionSpeed(2000);
	oInLinks[3].SetConnectionSpeed(2000);

	oOutLinks[6].SetConnectionSpeed(2000);
	oInLinks[6].SetConnectionSpeed(2000);*/
	
	//Make Connections
	//Destination.LinkToDevice(Source)
	//Source.LinkFrom(Destination)

	ConnectDevices(oTCPSource1,oMultiPathRouter1,&oOutLinks[0], &oInLinks[0], 0, 0);
	ConnectDevices(oMultiPathRouter1,oSinglePathRouter1, &oOutLinks[1], &oInLinks[1], 1, 0);
	ConnectDevices(oMultiPathRouter1,oSinglePathRouter2, &oOutLinks[2], &oInLinks[2], 2, 0);
	ConnectDevices(oMultiPathRouter1,oSinglePathRouter3, &oOutLinks[3], &oInLinks[3], 3, 0);
	ConnectDevices(oSinglePathRouter1,oDestination1, &oOutLinks[4], &oInLinks[4], 1, 0);
	ConnectDevices(oSinglePathRouter2,oDestination1, &oOutLinks[5], &oInLinks[5], 1, 1);
	ConnectDevices(oSinglePathRouter3,oDestination1, &oOutLinks[6], &oInLinks[6], 1, 2);
	




	//Build routing table for router 1
	oMultiPathRouter1.SetRoute(0,0,0);
	oMultiPathRouter1.SetRoute(1,1,0);
	//oMultiPathRouter1.SetRoute(1,2,1);
	//oMultiPathRouter1.SetRoute(1,3,2);
	
	oSinglePathRouter1.SetRoute(0,0,0);
	oSinglePathRouter1.SetRoute(1,1,0);
		
	oSinglePathRouter2.SetRoute(0,0,0);
	oSinglePathRouter2.SetRoute(1,1,0);

	oSinglePathRouter3.SetRoute(0,0,0);
	oSinglePathRouter3.SetRoute(1,1,0);
	
	oTCPSource1.GenerateFile(15000000);

	while (!(oTCPSource1.Done() && oTCPSource1.Done()))
	{
		oTCPSource1.Go();
		for (int i = 0; i < NUM_OF_LINKS; i++)
		{
			oOutLinks[i].Go();
			oInLinks[i].Go();
		}
	}
	oTCPSource1.Report();
	oMultiPathRouter1.Report();
	
	return 0;
}

void ConnectDevices(CComponent & oSourceDevice, CComponent & oDestinationDevice, CLink* pOutLink, CLink * pInLink, int nSourceDevice, int nDestinationDevice)
{
	oSourceDevice.LinkToDevice(pInLink,nSourceDevice);
	oSourceDevice.LinkFromDevice(pOutLink,nSourceDevice);

	oDestinationDevice.LinkToDevice(pOutLink,nDestinationDevice);
	oDestinationDevice.LinkFromDevice(pInLink,nDestinationDevice);
}

