// Packet.cc: implementation of the CPacket class.
//
//////////////////////////////////////////////////////////////////////

#include "Packet.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPacket::CPacket()
{
	m_nTotalTimeInSystem = -1;
	m_nPriorityClass = -1;
	m_nProcessingCounter = -1;
	m_nArrivalTime = -1;
	m_nPacketSize = -1;
	m_bProcessed = false;
	m_bAccepted = false;
	m_nTimeInQueue = -1;
	m_nTimeOutQueue = -1;
	m_nTimeStartProcessing = -1;
	m_nTimeFinishProcessing = -1;
	m_sPathHistory = "";
	m_bTransmitted = false;
	m_bAcknowledged = false;
	m_nSequenceNumber = 0;
	m_nSessionID = 0;
	m_nType = DATA;
	m_nRoutedClass = -1;
	m_nTimeOutCounter = 69;
}

CPacket::CPacket(const CPacket & roNewPacket)
{
	m_nTotalTimeInSystem = roNewPacket.TotalTimeInSystem();
	m_nPriorityClass = roNewPacket.PriorityClass();
	m_nProcessingCounter = roNewPacket.ProcessingRequirement();
	m_nArrivalTime = roNewPacket.ArrivalTime();
	m_nPacketSize = roNewPacket.PacketSize();
	m_nTotalTimeInQueue = roNewPacket.TotalTimeInQueue();
	m_nTotalProcessingTime = roNewPacket.TotalProcessingTime();
	m_bProcessed = roNewPacket.m_bProcessed;
	m_bAccepted = roNewPacket.m_bAccepted;	
	m_nTimeInQueue = roNewPacket.m_nTimeInQueue;
	m_nTimeOutQueue = roNewPacket.m_nTimeOutQueue;
	m_nTimeStartProcessing = roNewPacket.m_nTimeStartProcessing;
	m_nTimeFinishProcessing = roNewPacket.m_nTimeFinishProcessing;
	m_sPathHistory = "";
	m_bTransmitted = false;
	m_bAcknowledged = false;
	m_nSequenceNumber = 0;
	m_nSessionID = 0;
	m_nType = DATA;
	m_nRoutedClass = -1;
	m_nTimeOutCounter = 69;
}

CPacket::CPacket(int nNewArrivalTime, double nNewPacketSize, int nNewPriorityClass, int nNewPacketType)
{
	m_nPriorityClass = nNewPriorityClass;
	m_nArrivalTime = nNewArrivalTime;
	m_nPacketSize = nNewPacketSize;
	m_nProcessingCounter = 0;
	m_nTotalTimeInSystem = 0;
	m_nTotalTimeInQueue = 0;
	m_nTotalProcessingTime = 0;
	m_bProcessed = false;
	m_bAccepted = false;
	m_nTimeInQueue = -1;
	m_nTimeOutQueue = -1;
	m_nTimeStartProcessing = -1;
	m_nTimeFinishProcessing = -1;
	m_nType = nNewPacketType;
	m_bTransmitted = false;
	m_bAcknowledged = false;
	m_nSequenceNumber = 0;
	m_nSessionID = 0;
	m_nType = DATA;
	m_nRoutedClass = -1;
	m_sPathHistory = "";
	m_nTimeOutCounter = 69;
}

CPacket::~CPacket()
{

}

//////////////////////////////////////////////////////////////////////
// Overridden - custom operators
//////////////////////////////////////////////////////////////////////

CPacket & CPacket::operator=(const CPacket & roNewPacket)
{
	if (this != &roNewPacket)
    {
		m_nSequenceNumber       = roNewPacket.m_nSequenceNumber;
		m_nData                 = roNewPacket.m_nData;
		m_nTimeInQueue			= roNewPacket.m_nTimeInQueue;
		m_nTimeOutQueue			= roNewPacket.m_nTimeOutQueue;
		m_nTimeStartProcessing	= roNewPacket.m_nTimeStartProcessing;
		m_nTimeFinishProcessing	= roNewPacket.m_nTimeFinishProcessing;
		m_bAccepted				= roNewPacket.m_bAccepted;
		m_bProcessed			= roNewPacket.m_bProcessed;
		m_nTotalProcessingTime	= roNewPacket.m_nTotalProcessingTime;
		m_nTotalTimeInQueue		= roNewPacket.m_nTotalTimeInQueue;	
		m_nTotalTimeInSystem	= roNewPacket.m_nTotalTimeInSystem;	
		m_nPriorityClass		= roNewPacket.m_nPriorityClass;		
		m_nProcessingCounter	= roNewPacket.m_nProcessingCounter;	
		m_nType					= roNewPacket.m_nType;				
		m_nPacketTime			= roNewPacket.m_nPacketTime;
		m_nDestination			= roNewPacket.m_nDestination;
		m_nRoutedClass			= roNewPacket.m_nRoutedClass; 
		m_sPathHistory			= roNewPacket.m_sPathHistory;
		m_nSource				= roNewPacket.m_nSource;
		m_nTimeOutCounter		= roNewPacket.m_nTimeOutCounter;
		m_nSessionID			= roNewPacket.m_nSessionID;
		m_bAcknowledged			= roNewPacket.m_bAcknowledged;
		m_bTransmitted			= roNewPacket.m_bTransmitted;
		m_nArrivalTime			= roNewPacket.m_nArrivalTime;
		m_nPacketSize			= roNewPacket.m_nPacketSize;
    }
    return *this;
}


bool CPacket::operator==(const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime == nTimeHack);
}

bool CPacket::operator!=(const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime != nTimeHack);
}

bool CPacket::operator<=(const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime <= nTimeHack);
}

bool CPacket::operator>=(const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime >= nTimeHack);
}

bool CPacket::operator< (const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime < nTimeHack);
}

bool CPacket::operator> (const double & nTimeHack)
//Compares Arrival Times
{
	return (m_nArrivalTime > nTimeHack);
}

bool CPacket::operator==(const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime == roPacket.ArrivalTime());
}

bool CPacket::operator!=(const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime != roPacket.ArrivalTime());
}

bool CPacket::operator<=(const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime <= roPacket.ArrivalTime());
}

bool CPacket::operator>=(const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime >= roPacket.ArrivalTime());
}

bool CPacket::operator< (const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime < roPacket.ArrivalTime());
}

bool CPacket::operator> (const CPacket & roPacket)
//Compares Arrival Times
{
	return (m_nArrivalTime > roPacket.ArrivalTime());
}

//////////////////////////////////////////////////////////////////////
// Access to private - protected members
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
//  Set Values
/////////////////////////////////////////////////////////////////////
void CPacket::SetProcessed(bool bFlag)
{
	m_bProcessed = bFlag;
}

void CPacket::SetAccepted(bool bAccepted)
{
	m_bAccepted = bAccepted;
}

void CPacket::SetTransmitted(bool bTransmitted)
{
	m_bTransmitted = bTransmitted;
}

void CPacket::SetAcknowledged(bool bAcknowledged)
{
	m_bAcknowledged = bAcknowledged;
}

void CPacket::SetArrivalTime(int nNewArrivalTime)
{
	if (nNewArrivalTime > 0)
	{
		m_nArrivalTime = nNewArrivalTime;
	}
	else
	{
		m_nArrivalTime = 0;
	}
}

void CPacket::SetTimeInQueue(int nNewTime)
{
	if (nNewTime > 0)
	{
		m_nTimeInQueue = nNewTime;
	}
	else
	{
		m_nTimeInQueue = 0;
	}
}
void CPacket::SetTimeOutQueue(int nNewTime)
{
	if (m_nTimeOutQueue == -1)
	{
		if (nNewTime > m_nTimeInQueue)
		{
			m_nTimeOutQueue = nNewTime;
		}
		else
		{
			m_nTimeOutQueue = m_nTimeInQueue;
		}
	}
}
void CPacket::SetTimeStartProcessing(int nNewTime)
{
	if (m_nTimeStartProcessing == -1)
	{
		if (nNewTime > m_nTimeOutQueue)
		{
			m_nTimeStartProcessing = nNewTime;
		}
		else
		{
			m_nTimeStartProcessing = m_nTimeOutQueue;
		}
	}
}
void CPacket::SetTimeFinishProcessing(int nNewTime)
{
	if (nNewTime > m_nTimeStartProcessing)
	{
		m_nTimeFinishProcessing = nNewTime;
	}
	else
	{
		m_nTimeFinishProcessing = m_nTimeStartProcessing;
	}
}

void CPacket::SetProcessingRequirement(int nNewProcessingCounter)
{
	if (nNewProcessingCounter > 0)
	{
		m_nProcessingCounter = nNewProcessingCounter;
	}
	else
	{
		m_nProcessingCounter = 0;
	}
}

void CPacket::IncrementTotalTime(int nTimeDelta)
{
	if ( nTimeDelta > 0 )
	{
		m_nTotalTimeInSystem += nTimeDelta;
	}
}

void CPacket::IncrementQueueTime(int nTimeDelta)
{
	if ( nTimeDelta > 0 )
	{
		m_nTotalTimeInQueue += nTimeDelta;
		IncrementTotalTime(nTimeDelta);
	}
}

void CPacket::IncrementProcessingTime(int nTimeDelta)
{
	if ( nTimeDelta > 0 )
	{
		m_nTotalProcessingTime += nTimeDelta;
		IncrementTotalTime(nTimeDelta);
	}
}

void CPacket::DecrementProcessingRequirement(int nTimeDelta)
{
	if ( nTimeDelta > 0 )
	{
		m_nProcessingCounter -= nTimeDelta;
	}
}

void CPacket::SetPriorityClass(int nNewPriorityClass)
{
	if ((nNewPriorityClass >= 0) && (nNewPriorityClass <= MAX_PRIORITY))
	{
		m_nPriorityClass = nNewPriorityClass;
	}
}

void CPacket::SetType(int nNewType)
{
	if ((nNewType >= 0) && (nNewType <= MAX_TYPE))
	{
		m_nType = nNewType;
	}
}

void CPacket::ResetProcessingRequirement()
{
	m_nProcessingCounter = m_nPacketSize;
}

void CPacket::SetDestination(int nDestination)
{
	m_nDestination = nDestination;
}

void CPacket::SetRoutedClass(int nClass)
{
	m_nRoutedClass = nClass;
}

void CPacket::SetData(int nData)
{
	m_nData = nData;
}

void CPacket::SetSequenceNumber(int nSequenceNumber)
{
	m_nSequenceNumber = nSequenceNumber;
}

void CPacket::UpdateClass()
{
	m_nPriorityClass = m_nRoutedClass;
}

void CPacket::AppendHistory(string sPath)
{
	m_sPathHistory = m_sPathHistory + "  " + sPath;
}

void CPacket::SetTimeOutCounter(int nTimeOut)
{
	m_nTimeOutCounter = nTimeOut;
}

void CPacket::SetSource(int nSource)
{
	m_nSource = nSource;
}

void CPacket::SetSessionID(int nSession)
{
	m_nSessionID = nSession;
}

void CPacket::DecrementTimeOutCounter()
{
	m_nTimeOutCounter--;
	if (m_nTimeOutCounter < 0)
	{
		m_nTimeOutCounter = 0;
	}
}

/////////////////////////////////////////////////////////////////////
//  Get Values
/////////////////////////////////////////////////////////////////////

int CPacket::ArrivalTime() const
{
	return m_nArrivalTime;
}

int CPacket::Destination() const
{
	return m_nDestination;
}

int CPacket::PacketTime() const
{
	if (m_nType == ARRIVAL)
	{
		return m_nArrivalTime;
	}
	else if (m_nType == DEPARTURE)
	{
		return m_nTimeFinishProcessing;
	}
	else
	{
		return m_nPacketTime;
	}
}

double CPacket::PacketSize() const
{
	return m_nPacketSize;
}

int CPacket::ProcessingRequirement() const
{
	return m_nProcessingCounter;
}

int CPacket::TotalTimeInSystem() const
{
	return m_nTimeFinishProcessing - m_nTimeInQueue;
}
int CPacket::TotalTimeInQueue() const
{
	return max(0,(m_nTimeOutQueue - m_nArrivalTime) - m_nTotalProcessingTime);
}
int CPacket::TimeInQueue() const
{
	return m_nTimeInQueue;
}
int CPacket::TimeOutQueue() const
{
	return m_nTimeOutQueue;
}
int CPacket::TimeStartProcessing() const
{
	return m_nTimeStartProcessing;
}
int CPacket::TimeFinishProcessing() const
{
	return m_nTimeFinishProcessing;
}
int CPacket::TotalProcessingTime() const
{
	return m_nTotalProcessingTime;
}

int CPacket::PriorityClass() const
{
	return m_nPriorityClass;
}

int CPacket::Type() const
{
	return m_nType;
}

bool CPacket::IsProcessed()
{
	return m_bProcessed;
}

bool CPacket::Accepted()
{
	return m_bAccepted;
}

bool CPacket::Transmitted() const
{
	return m_bTransmitted;
}

bool CPacket::Acknowledged()
{
	return m_bAcknowledged;
}

int CPacket::TimeOutCounter() const
{
	return m_nTimeOutCounter;
}

int CPacket::Source() const
{
	return m_nSource;
}
int CPacket::Data() const
{
	return m_nData;
}

int CPacket::SequenceNumber() const
{
	return m_nSequenceNumber;
}
int CPacket::SessionID() const
{
	return m_nSessionID;
}

