// Packet.h: interface for the CPacket class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(PACKET_H)
#define PACKET_H

#include "const.h"
#include "DPIUtilities.h"

class CPacket  
{
public:
	
	bool Acknowledged();
	bool Accepted();
	bool IsProcessed();
	//Construction / Destruction
	CPacket();
	CPacket(const CPacket & roNewPacket);
	CPacket(int nNewArrivalTime, double nNewPacketSize, int nNewPriorityClass, int nNewPacketType);
	virtual ~CPacket();

	//Overloaded - custom operators
	CPacket & operator=(const CPacket & pNewPacket);
	bool operator==(const double & nTimeHack);
	bool operator!=(const double & nTimeHack);
	bool operator<=(const double & nTimeHack);
	bool operator>=(const double & nTimeHack);
	bool operator< (const double & nTimeHack);
	bool operator> (const double & nTimeHack);
	bool operator==(const CPacket & roPacket);
	bool operator!=(const CPacket & roPacket);
	bool operator<=(const CPacket & roPacket);
	bool operator>=(const CPacket & roPacket);
	bool operator< (const CPacket & roPacket);
	bool operator> (const CPacket & roPacket);

	//Access to private - protected members
	//Set Values
	void SetProcessed(bool bFlag = true);
	void SetAccepted(bool bAccepted);
	void SetAcknowledged(bool bAcknowledged);
	void SetArrivalTime(int nNewArrivalTime);
	void SetTimeInQueue(int nNewTime);
	void SetTimeOutQueue(int nNewTime);
	void SetTimeStartProcessing(int nNewTime);
	void SetTimeFinishProcessing(int nNewTime);
	void SetProcessingRequirement(int nNewProcessingCounter);
	void IncrementTotalTime(int nTimeDelta = 1);
	void IncrementQueueTime(int nTimeDelta = 1);
	void IncrementProcessingTime(int nTimeDelta = 1);
	void DecrementProcessingRequirement(int nTimeDelta = 1);
	
	//Added for NetSim
	void SetPriorityClass(int nNewPriorityClass);
	void SetType(int nNewType);
	void ResetProcessingRequirement();
	void SetDestination(int nDestination);
	void SetRoutedClass(int nClass);
	void UpdateClass();
	void AppendHistory(string sPath);
	void SetTimeOutCounter(int nTimeOut);
	void DecrementTimeOutCounter();
	void SetSource(int nSource);
	void SetSessionID(int nSource);
	void SetData(int nData);
	void SetSequenceNumber(int nSequenceNumber);
	void SetTransmitted(bool bTransmitted);

	//Get Values
	int ArrivalTime() const;
	double PacketSize() const;
	int ProcessingRequirement() const;
	int TotalTimeInSystem() const;
	int TotalTimeInQueue() const;
	int TimeInQueue() const;
	int TimeOutQueue() const;
	int TimeStartProcessing() const;
	int TimeFinishProcessing() const;
	int TotalProcessingTime() const;
	int PriorityClass() const;
	int Type() const;
	int PacketTime() const;
	int Destination() const;
	int Source() const;
	int TimeOutCounter() const;
	int SessionID() const;
	int Data() const;
	int SequenceNumber() const;
	bool Transmitted() const;
	
protected:
	int m_nSequenceNumber;
	int m_nData;
	int m_nTimeInQueue;
	int m_nTimeOutQueue;
	int m_nTimeStartProcessing;
	int m_nTimeFinishProcessing;
	bool m_bAccepted;
	bool m_bProcessed;
	int m_nTotalProcessingTime; //Tracks processing time
	int m_nTotalTimeInQueue;	//Tracks time in queue 
	int m_nTotalTimeInSystem;	//Tracks time in queue and processing time
	int m_nPriorityClass;		//The priority of this Packet
	int m_nProcessingCounter;	//The number of "ticks" this Packet will need to process
	int m_nType;				//Arrival or Departure Packet Type
	int m_nPacketTime;
	int m_nDestination;
	int m_nRoutedClass; 
	string m_sPathHistory;
	int m_nSource;
	int m_nTimeOutCounter;
	int m_nSessionID;
	bool m_bAcknowledged;
	bool m_bTransmitted;
	

	
	//Values will be generated
	//using GSL library
	int m_nArrivalTime;		//The hack within the time interval that this Packet arrives
	double m_nPacketSize;	//The size of the response to this Packet
	




};

#endif



