// PacketGenerator.cpp: implementation of the CPacketGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "PacketGenerator.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPacketGenerator::CPacketGenerator(int nAverageSize, int nPacketsPerInterval, int nTotalTime)
{
	m_nAveragePacketSize = nAverageSize;
	m_nPacketsPerInterval = nPacketsPerInterval;
	m_nLocalTime = 0;
	m_nTimeRemaining = nTotalTime;
}

CPacketGenerator::~CPacketGenerator()
{

}

/////////////////////////////////////////////////////////////////////
// Access member variables
/////////////////////////////////////////////////////////////////////

void CPacketGenerator::SetAveragePacketSize(int nAverageSize)
{
	if (nAverageSize > 0)
	{
		m_nAveragePacketSize = nAverageSize;
	}
}

void CPacketGenerator::SetPacketsPerInterval(int nPacketsPerInterval)
{
	if (nPacketsPerInterval > 0)
	{
		m_nPacketsPerInterval = nPacketsPerInterval;
	}
}

int CPacketGenerator::AveragePacketSize()
{
	return m_nAveragePacketSize;
}

int CPacketGenerator::PacketsPerInterval()
{
	return m_nPacketsPerInterval;
}


////////////////////////////////////////////////////////////////////
// Set Connection
////////////////////////////////////////////////////////////////////

bool CPacketGenerator::ConnectToDevice(CElement* pElement, int nLabel)
{
	return m_oToDevice.SetConnection(pElement, nLabel);
}

////////////////////////////////////////////////////////////////////
// Activate Generator
////////////////////////////////////////////////////////////////////

void CPacketGenerator::Go()
{
	int h = 0;
	m_nLocalTime++;
	if (m_nTimeRemaining > 0)
	{

		m_nTimeRemaining--;
		//Create random number generator using GSL
		gsl_rng * r = gsl_rng_alloc (gsl_rng_taus);
		//seed the generator with the clock vs static number so it is different each run
		gsl_rng_set (r, clock());

		for (int i = 0; i < m_nPacketsPerInterval; i++)
		{
			int nSize = int( ceil(gsl_ran_exponential (r, m_nAveragePacketSize)) );
			CPacket * pNewPacket = new CPacket(0, nSize,0,0);
			pNewPacket->SetDestination(1);
			pNewPacket->ResetProcessingRequirement();
			pNewPacket->SetArrivalTime(m_nLocalTime);
			pNewPacket->SetTimeInQueue(m_nLocalTime);
			pNewPacket->SetPriorityClass(h);
			h++;
			if (h==3)
			{
				h = 0;
			}
			m_oToDevice.Push(pNewPacket);
		}
	}
}

bool CPacketGenerator::Done()
{
	return (m_nTimeRemaining == 0);
}


