// RouteSelector.cpp: implementation of the CRouteSelector class.
//
//////////////////////////////////////////////////////////////////////

#include "RouteSelector.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRouteSelector::CRouteSelector(int nNumOfDevices, int nType)
{
	m_nType = nType;
	m_vConnectedDevices.resize(nNumOfDevices);
	m_vPressureLimits.resize(nNumOfDevices);
	for (int i = 0; i < nNumOfDevices; i++)
	{
		CPushConnection* nNewConnection = new CPushConnection;
		m_vConnectedDevices[i] = nNewConnection;
		m_vPressureLimits[i] = 100;
	}
	
	m_nNumOfDevices = nNumOfDevices;
}

CRouteSelector::~CRouteSelector()
{

}

void CRouteSelector::SetNumberOfDevices(int nNumberOfDevices)
{
	for (int i = 0; i < m_nNumOfDevices; i++)
	{
		if (m_vConnectedDevices[i] != NULL)
		{
			delete m_vConnectedDevices[i];
		}
	}
	m_vConnectedDevices.resize(nNumberOfDevices);
	m_vPressureLimits.resize(nNumberOfDevices);

	for (i = 0; i < nNumberOfDevices; i++)
	{
		CPushConnection* nNewConnection = new CPushConnection;
		m_vConnectedDevices[i] = nNewConnection;
		m_vPressureLimits[i] = 100;
	}
	m_nNumOfDevices = nNumberOfDevices;

	m_oSimpleRoutingTable.Reset();
	m_oRRMultiPathRoutingTable.Reset();
	m_oSelectiveMultiPathRoutingTable.Reset();
}

bool CRouteSelector::ConnectToSink(CElement* pElement, int nLabel)
{
	return m_oBadRouteSink.SetConnection(pElement, nLabel);
}

bool CRouteSelector::ConnectToDevice(CElement* pElement, int nLabel, int nDevice)
{
	if ((nDevice >= 0) && (nDevice <= m_nNumOfDevices))
	{
		return m_vConnectedDevices[nDevice]->SetConnection(pElement, nLabel);
	}
	else
	{
		LOG("CRouteSelector::ConnectToDevice");
		return false;
	}	
}

bool CRouteSelector::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	int nDevice;
	CRoute oRoute;
	bool bNoRoute = true;
	int i = 0;
	int nLastDevice = -1;

	switch (m_nType)
	{
	case 0:
		nDevice = m_oSimpleRoutingTable.GetDevice(pIncommingPacket->Destination());
		break;
	case 1:
		nDevice = m_oRRMultiPathRoutingTable.GetDevice(pIncommingPacket->Destination());
		pIncommingPacket->SetPriorityClass(m_oRRMultiPathRoutingTable.GetLastPriority(pIncommingPacket->Destination()));
		break;
	case 2:
		while (bNoRoute)
		{
			oRoute = m_oSelectiveMultiPathRoutingTable.GetDevice(pIncommingPacket->Destination(), i);
			if (tooMuchPressure(oRoute.nOutputDevice) && (oRoute.nPriority != -1))
			{
				i++;
				bNoRoute = true;
			}
			else if (tooMuchPressure(oRoute.nOutputDevice))
			{
				oRoute = m_oSelectiveMultiPathRoutingTable.GetDevice(pIncommingPacket->Destination(), 0);
				bNoRoute = false;
			}
			else
			{
				bNoRoute = false;
			}

		}
		nDevice = oRoute.nOutputDevice;
		pIncommingPacket->SetPriorityClass(oRoute.nPriority);
		break;



	default:
		nDevice = m_oSimpleRoutingTable.GetDevice(pIncommingPacket->Destination());
		break;
	};

	if (nDevice == -1) //Bad route
	{
		return m_oBadRouteSink.Push(pIncommingPacket);
	}
	else
	{
		return m_vConnectedDevices[nDevice]->Push(pIncommingPacket);
	}
}

void CRouteSelector::SetRoute(int nDestination, int nDevice, int nPriority)
{
	switch (m_nType)
	{
	case 0:
		m_oSimpleRoutingTable.SetRoute(nDestination, nDevice, nPriority);
		break;
	case 1:
		m_oRRMultiPathRoutingTable.SetRoute(nDestination, nDevice, nPriority);
		break;
	case 2:
		m_oSelectiveMultiPathRoutingTable.SetRoute(nDestination, nDevice, nPriority);
		break;
	default:
		m_oSimpleRoutingTable.SetRoute(nDestination, nDevice, nPriority);
		break;
	};
}

void CRouteSelector::SetPressureLimit(int nDevice, int nLimit)
{
	m_vPressureLimits[nDevice] = nLimit;
}

void CRouteSelector::SetType(int nType)
{
	m_nType = nType;
}


bool CRouteSelector::tooMuchPressure(int nDevice)
{
	CDropTailQueue* nQueue = (CDropTailQueue*)m_vConnectedDevices[nDevice]->Element();
	return (nQueue->BackPressure() >= m_vPressureLimits[nDevice]);
}
