// SinglePathRouter.cpp: implementation of the CRouter class.
//
//////////////////////////////////////////////////////////////////////

#include "Router.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRouter::CRouter(string sLabel, int nNumberOfDevices, int nType, int nNetAddress):CStoreAndForwardRepeater(sLabel, nNetAddress)
{
	m_nType = nType;
	m_sLabel = sLabel;
	m_oRouteSelector.SetNumberOfDevices(nNumberOfDevices);
	m_oRouteSelector.SetType(nType);
	m_nNumberOfDevices = nNumberOfDevices;

	m_oEth.resize(nNumberOfDevices);
	m_oEthQueue.resize(nNumberOfDevices);
	m_oEthSink.resize(nNumberOfDevices);
	


	for (int i = 0; i < nNumberOfDevices; i++)
	{
		m_oEth[i].ConnectToSystem(&m_oRouteSelector, 0);
		m_oEthQueue[i].ConnectToSink(&m_oEthSink[i], 0);
		m_oRouteSelector.ConnectToDevice(&m_oEthQueue[i],0,i);
		m_oEth[i].ConnectToQueue(&m_oEthQueue[i], 1);
		m_oEth[i].SetParentLabel(m_sLabel);
		m_oEth[i].SetLabel("Eth" + str(i));
		m_oEthQueue[i].SetParentLabel(m_sLabel);
		m_oEthQueue[i].SetLabel("EthQueue" + str(i));
		m_oRouteSelector.ConnectToSink(&m_oBadRouteSink, 0);
		m_oEthSink[i].SetParentLabel(m_sLabel);
		m_oEthSink[i].SetParentAddress(m_nNetAddress);
	}
}

CRouter::~CRouter()
{

}

bool CRouter::LinkToDevice(CLink* pLink, int nDevice)
{
	if (nDevice < m_nNumberOfDevices)
	{
		return pLink->ConnectToDevice(&m_oEth[nDevice], 0);
	}
	else
	{
		LOG("CRouter::LinkToDevice");
		return false;
	}
}

bool CRouter::LinkFromDevice(CLink* pLink, int nDevice)
{
	if (nDevice < m_nNumberOfDevices)
	{
		pLink->ConnectFromDevice(&m_oEth[nDevice]);
		return m_oEth[nDevice].ConnectToNetwork(pLink, 0);
	}
	else
	{
		LOG("CRouter::LinkFromDevice");
		return false;
	}
}

void CRouter::SetRoute(int nDestination, int nDevice, int nPriority)
{
	m_oRouteSelector.SetRoute(nDestination, nDevice, nPriority);
}

void CRouter::SetPressureLimit(int nDevice, int nLimit)
{
	if (m_nType == 2)
	{
		m_oRouteSelector.SetPressureLimit(nDevice,nLimit);
	}
}

void CRouter::Report()
{
	for (int i = 0; i < m_nNumberOfDevices; i++)
	{
		m_oEth[i].Report();
	}
}

