// TCPSink.cpp: implementation of the CTCPSink class.
//
//////////////////////////////////////////////////////////////////////

#include "TCPSink.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTCPSink::CTCPSink(string sLabel):CElement(sLabel)
{
	
	m_nPacketCount = 0;
	m_nNextExpected = 0;
}

CTCPSink::~CTCPSink()
{

}

bool CTCPSink::ConnectToNetwork(CElement* pElement, int nLabel)
{
	return m_oNetwork.SetConnection(pElement, nLabel);
}

bool CTCPSink::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	if ((pIncommingPacket->Destination() != m_nParentNetAddress) && (m_nParentNetAddress != -1))
	{
		LOG("Bad Routing at:" + m_sParentLabel + " - " + m_sLabel);
	}

	if (nConnectionLabel == 0) //Packet pushed on to device from network
	{
		CPacket* pACKPacket = new CPacket(0, MIN_PACKET_SIZE,0,0);
		pACKPacket->SetSource(pIncommingPacket->Destination());
		pACKPacket->SetDestination(pIncommingPacket->Source());
		pACKPacket->SetSessionID(pIncommingPacket->SessionID());
		pACKPacket->SetPriorityClass(MAX_PRIORITY);
		if (pIncommingPacket->SequenceNumber() == m_nNextExpected)
		{
			m_nNextExpected++;
			pACKPacket->SetType(ACK);
		}
		else
		{
			pACKPacket->SetType(DUPACK);
		}

		pACKPacket->SetData(pIncommingPacket->SequenceNumber());


		pIncommingPacket->AppendHistory("RCVD:" + m_sParentLabel + "-" + m_sLabel);
		m_nPacketCount++;
		delete pIncommingPacket;
		return m_oNetwork.Push(pACKPacket);

	}
	else
	{
		LOG("CTCPSink::AcceptPacket");
		return false; //This Element has no other push connections
	}
}

void CTCPSink::Reset()
{
	int m_nPacketCount = 0;
	int m_nNextExpected = 0;
}
