// TCPSource.cpp: implementation of the CTCPSource class.
//
//////////////////////////////////////////////////////////////////////

#include "TCPSource.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTCPSource::CTCPSource(string sLabel, int nSourceAddress, int nDestinationAddress, int nSize): CElement(sLabel)
{
	m_bProcessingFile = false;
	m_nSessionID = -1;
	m_nDestination = nDestinationAddress;
	m_nWindowSize = nSize;
	m_nMaxWindowSize = nSize;
	m_nDropCount = 0;
	m_nAckCount = 0;
	m_nLowestAck = -1;
	m_nWindowStart = 0;
	m_nSourceAddress = nSourceAddress;
	m_nTotalTime=0;
}

CTCPSource::~CTCPSource()
{

}

bool CTCPSource::ConnectToNetwork(CElement* pElement, int nLabel)
{
	return m_oNetwork.SetConnection(pElement, nLabel);
}

void CTCPSource::GenerateFile(int nFileSize)
{
	if (!m_bProcessingFile)
	{
		m_bProcessingFile = true;
		int nSize;
		int nBytesRemaining = nFileSize;
		//int nSequenceNumber = 0;
		m_nTotalPackets = (nFileSize / MAX_PACKET_SIZE);
		m_nSessionID++;
		m_vPackets.resize(m_nTotalPackets+1);
		for (int i = 0; i <= m_nTotalPackets; i++)	
		{
			
			nSize = min(MAX_PACKET_SIZE, nBytesRemaining);
			
			nBytesRemaining -= nSize;
			CPacket* pNewPacket = new CPacket(0, nSize,0,0);
			pNewPacket->SetDestination(m_nDestination);
			pNewPacket->ResetProcessingRequirement();
			//nSequenceNumber++;
			pNewPacket->SetSequenceNumber(i);
			pNewPacket->SetSource(m_nSourceAddress);
			pNewPacket->SetSessionID(m_nSessionID);
			m_vPackets[i] = pNewPacket;
		}
	}
}

bool CTCPSource::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	if ((pIncommingPacket->Type() == ACK) && (pIncommingPacket->SessionID() == m_nSessionID) ||
		(pIncommingPacket->Type() == DUPACK) && (pIncommingPacket->SessionID() == m_nSessionID))
	{
		
		if (pIncommingPacket->Type() == DUPACK)
		{
			m_nDupCount++;
			if (m_nAckCount >= DUP_THRESH)
			{
				m_nDropCount = 0;
				m_nAckCount = 0;
				m_nDupCount = 0;
				m_nWindowSize = max(min(m_nWindowSize * 2, m_nMaxWindowSize),1);
				fastRetransmit();
			}
			
		}
		else
		{
			m_nAckCount++;
			if (m_nAckCount >= ACK_THRESH)
			{
				m_nDropCount = 0;
				m_nAckCount = 0;
				m_nWindowSize = max(min(m_nWindowSize + 1, m_nMaxWindowSize),1);
			}
		}
		m_vPackets[pIncommingPacket->Data()]->SetAcknowledged(true);
		delete pIncommingPacket;
		while (m_vPackets[m_nLowestAck + 1]->Acknowledged())
		{
			if (m_nLowestAck < m_nTotalPackets)
			{
				m_nLowestAck++;
				m_nWindowStart++;
				if (m_nLowestAck == m_nTotalPackets)
				{
					reset();
					return true;
				}
			}
		}
	}
	else
	{
		delete pIncommingPacket;
	}
	return true;
}

void CTCPSource::reset()
{
	for (int i = 0; i < m_vPackets.size(); i++)
	{
		delete m_vPackets[i];
	}
	m_nWindowStart = 0;
	m_nWindowSize = m_nMaxWindowSize;
	m_nLowestAck = 0;
	m_bProcessingFile = false;
	m_nSessionID = -1;
}

void CTCPSource::Go()
{
	
	CPacket* pOutGoingPacket = NULL;
	if (m_bProcessingFile)
	{
		m_nTotalTime++;
		for (int i = m_nWindowStart; i <= (min((m_nWindowStart + m_nWindowSize), m_nTotalPackets)); i++)
		{
			if (!m_vPackets[i]->Transmitted())
			{
				m_vPackets[i]->SetTimeOutCounter(TCP_TIMEOUT);
				pOutGoingPacket = new CPacket;
				*pOutGoingPacket = *m_vPackets[i];
				if (pOutGoingPacket->SequenceNumber() == 1332)
				{
					int g = 5;
				}
				m_oNetwork.Push(pOutGoingPacket);
				m_vPackets[i]->SetTransmitted(true);
			}
			else if (!m_vPackets[i]->Acknowledged())
			{
				m_vPackets[i]->DecrementTimeOutCounter();
				if (m_vPackets[i]->TimeOutCounter() <= 0)
				{
					m_vPackets[i]->SetTimeOutCounter(TCP_TIMEOUT);
					pOutGoingPacket = new CPacket;
					*pOutGoingPacket = *m_vPackets[i];
					m_oNetwork.Push(pOutGoingPacket);
					m_nDropCount++;
					if (m_nDropCount >= DROP_THRESH)
					{
						m_nDropCount = 0;
						m_nAckCount = 0;
						m_nWindowSize = (m_nWindowSize / 2) + 1;
					}
				}
			}
		}
	}
}

bool CTCPSource::Done()
{
	return (!m_bProcessingFile);
}

void CTCPSource::SetSource(int nSourceAddress)
{
	m_nSourceAddress = nSourceAddress;
}
void CTCPSource::SetDestination(int nDestinationAddress)
{
	m_nDestination = nDestinationAddress;
}

int CTCPSource::TotalTime()
{
	return m_nTotalTime;
}



void CTCPSource::fastRetransmit()
{
	for (int i = m_nWindowStart; i <= (min((m_nWindowStart + m_nWindowSize), m_nTotalPackets)); i++)
	{
		if (m_vPackets[i]->Transmitted() && (!m_vPackets[i]->Acknowledged()))
		{
			m_vPackets[i]->SetTransmitted(false);
		}
	}
}
