// TCPTrafficGenerator.cpp: implementation of the CTCPTrafficGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "TCPTrafficGenerator.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTCPTrafficGenerator::CTCPTrafficGenerator(string sLabel, int nSourceAddress, int nDestinationAddress, int nType):CComponent(sLabel, nSourceAddress)
{
	
	switch (nType)
	{
	case TCP:
		m_pTCPSource = new CTCPSource;
		break;
	case TCPPR:
		m_pTCPSource = new CTCPPRSource;
		break;
	default:
		m_pTCPSource = NULL;
	}
	
	m_oEth0.ConnectToSystem(m_pTCPSource,0);
	m_pTCPSource->ConnectToNetwork(&m_oEth0Queue,0);
	m_pTCPSource->SetSource(nSourceAddress);
	m_pTCPSource->SetDestination(nDestinationAddress);
	m_pTCPSource->SetParentLabel(sLabel);
	m_oEth0.ConnectToQueue(&m_oEth0Queue,1);
	m_oEth0Queue.ConnectToSink(&m_oEth0Sink,0);
	m_oEth0Queue.SetCapacity(INFINITY);
	m_nLocalTime = 0;
	m_nTotalTime = 0;
	m_oEth0.SetParentLabel(m_sLabel);
	m_oEth0.SetLabel("Eth0");
	m_oEth0Queue.SetParentLabel(m_sLabel);
	m_oEth0Queue.SetLabel("EthQueue0");
}

CTCPTrafficGenerator::~CTCPTrafficGenerator()
{
	if (m_pTCPSource)
	{
		delete m_pTCPSource;
	}
}

bool CTCPTrafficGenerator::LinkToDevice(CLink* pLink, int nDevice)
{
	if (nDevice == 0)
	{
		return pLink->ConnectToDevice(&m_oEth0, 0);
	}
	else
	{
		LOG("CTCPTrafficGenerator::LinkToDevice");
		return false;
	}
}

bool CTCPTrafficGenerator::LinkFromDevice(CLink* pLink, int nDevice)
{
	if (nDevice == 0)
	{
		pLink->ConnectFromDevice(&m_oEth0);
		return m_oEth0.ConnectToNetwork(pLink, 0);
	}
	else
	{
		LOG("CTCPTrafficGenerator::LinkFromDevice");
		return false;
	}
}

void CTCPTrafficGenerator::Go()
{
	m_pTCPSource->Go();
	m_nLocalTime++;
}

void CTCPTrafficGenerator::GenerateFile(int nFileSize)
{
	m_pTCPSource->GenerateFile(nFileSize);
	m_nFileSize = nFileSize;
}

bool CTCPTrafficGenerator::Done()
{
	return m_pTCPSource->Done();
}

void CTCPTrafficGenerator::Report()
{
	m_nTotalTime = max(1,m_pTCPSource->TotalTime());
	cout<<m_sLabel + " :\nTotal Time = " + str(m_nTotalTime) + "\n";
	double nThroughput = (m_nFileSize / (m_nTotalTime / 1000.00)/1000.00) ;
	cout<<"Throughput: " + str(nThroughput) + " Kbps\n";
}