// WTPScheduler.cpp: implementation of the CWTPScheduler class.
//
//////////////////////////////////////////////////////////////////////

#include "WTPScheduler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWTPScheduler::CWTPScheduler(int nNumberOfClasses)
{
	m_vQueue.resize(nNumberOfClasses);
	m_oClassifier.SetNumberOfClasses(nNumberOfClasses);
	m_oScheduler.SetNumberOfQueues(nNumberOfClasses);
	m_nNumberOfClasses = nNumberOfClasses;
	for (int i = 0; i < nNumberOfClasses; i++)
	{
		
		m_oClassifier.ConnectToClass(&m_vQueue[i], 0, i);
		m_oScheduler.ConnectToQueue(&m_vQueue[i], 1, i);
	
		m_vQueue[i].SetWeight(2 * (nNumberOfClasses - i));
	}
}

CWTPScheduler::~CWTPScheduler()
{

}

bool CWTPScheduler::ConnectToSink(CElement* pElement, int nLabel)
{
	bool bConnected = true;
	for (int i = 0; i < m_nNumberOfClasses; i++)
	{
		bConnected = (bConnected && m_vQueue[i].ConnectToSink(pElement, nLabel));
	}
	return bConnected;
}

bool CWTPScheduler::AcceptPacket(CPacket* pIncommingPacket, int nConnectionLabel)
{
	return (m_oClassifier.AcceptPacket(pIncommingPacket, nConnectionLabel));
}

CPacket* CWTPScheduler::GetPacket(int nConnectionLabel)
{
	return (m_oScheduler.GetPacket(nConnectionLabel));
}

int CWTPScheduler::BackPressure()
{
	int nTotalPressure = 0;
	int nTotalWeight = 0;
	int nPressure = 0;

	for (int i = 0; i < m_nNumberOfClasses; i++)
	{
		nTotalPressure += m_vQueue[i].BackPressure();
		nTotalWeight += m_vQueue[i].QueuePriority();
	}
	
	if (nTotalWeight > 0)
	{
		nPressure = nTotalPressure/nTotalWeight;
	}
	else
	{
		nPressure = 0;
	}

	return nPressure;

}

void CWTPScheduler::Go()
{
	for (int i = 0; i < m_nNumberOfClasses; i++)
	{
		m_vQueue[i].Go();
	}
}

bool CWTPScheduler::Done()
{
	bool bDone = true;
	for (int i = 0; i < m_nNumberOfClasses; i++)
	{
		bDone = (bDone && m_vQueue[i].Done());
	}
	return bDone;
}

void CWTPScheduler::SetNumberOfClasses(int nNumberOfClasses)
{
	m_nNumberOfClasses = nNumberOfClasses;
	m_vQueue.resize(nNumberOfClasses);
	m_oClassifier.SetNumberOfClasses(nNumberOfClasses);
	m_oScheduler.SetNumberOfQueues(nNumberOfClasses);

	for (int i = 0; i < nNumberOfClasses; i++)
	{
		
		m_oClassifier.ConnectToClass(&m_vQueue[i], 0, i);
		m_oScheduler.ConnectToQueue(&m_vQueue[i], 1, i);
		m_vQueue[i].SetWeight(pow(2,nNumberOfClasses-(i+1)));
	}
}

